//factory
#pragma once

#include <map>
#include <iostream>
#include <functional>
#include <memory>
#include "AclLiteResource.h"
#include "AclLiteApp.h"
#include "AclLiteThread.h"
#include "AclLiteType.h"
#include "VideoCapture.h"
#include "AclLiteUtils.h"
#include "CarParams.h"

#include "detectPreprocess.h"
#include "helmetPreprocess.h"
#include "mqttsend.h"
#include "classifyPreprocess.h"
#include "classifyPostprocess.h"
#include "../pushrtsp/pushrtspthread.h"
#include "onlyDetectInference.h"
#include "inference.h"
#include "detectPostprocess.h"
#include "centerfacePostprocess.h"



class GetInferenceInstance{
// 获得线程推理实例，不同算法的线程实例不同，例如 检测 + 分类的算法 参考carColorInferenceInstance 纯检测的参考helmetInferenceInstance
public:
    GetInferenceInstance();
    virtual void Init(const char* kConfigFile);
    virtual ~GetInferenceInstance();

    virtual void getInstance(std::vector<AclLiteThreadParam>& threadTbl, int32_t deviceId,aclrtContext& context, aclrtRunMode& runMode, int rtspNumPerDevice,bool kDisplay,int addrtspNumPerDevice=0);
    virtual bool readOutputTypeConfig(int channelId);

protected:
    const char* kConfigFile_;
    std::string kRtspUrl_;
public:
    ModelInfo modelInfo_;
    OnlyDetectNameInfo detectNameInfo_;
    Detect2clsNameInfo detectclsNameInfo_;

};
class carColorInferenceInstance : public GetInferenceInstance{
public:
    carColorInferenceInstance();
    void Init(const char* kConfigFile) override;
    ~carColorInferenceInstance();
    void getInstance(std::vector<AclLiteThreadParam>& threadTbl, int32_t deviceId,aclrtContext& context, aclrtRunMode& runMode, int rtspNumPerDevice,bool kDisplay, int addrtspNumPerDevice=0);
//    Detect2clsNameInfo nameInfo_;
//    ModelInfo carModelInfo_;

};
class helmetInferenceInstance : public GetInferenceInstance{
public:
    helmetInferenceInstance();
    void Init(const char* kConfigFile) override;
    ~helmetInferenceInstance();
    void getInstance(std::vector<AclLiteThreadParam>& threadTbl, int32_t deviceId,aclrtContext& context, aclrtRunMode& runMode, int rtspNumPerDevice,bool kDisplay, int addrtspNumPerDevice=0);
//    const std::string kDetectPreName_ = "helmetdetectPre";
//    const std::string kDetectPostName_ = "helmetdetectPost";
//    const std::string kInferName_ = "helmetinference";
//    ModelInfo helmetinfo_;
//    OnlyDetectNameInfo detectNameInfo_;

};

class fireSmokeInferenceInstance : public GetInferenceInstance{
public:
    fireSmokeInferenceInstance();
    void Init(const char* kConfigFile) override;
    ~fireSmokeInferenceInstance();
    void getInstance(std::vector<AclLiteThreadParam>& threadTbl, int32_t deviceId,aclrtContext& context, aclrtRunMode& runMode, int rtspNumPerDevice,bool kDisplay, int addrtspNumPerDevice=0);
//    const std::string kDetectPreName_ = "firesmokedetectPre";
//    const std::string kDetectPostName_ = "firesmokedetectPost";
//    const std::string kInferName_ = "firesmokeinference";
//    ModelInfo firesmokeinfo_;
//    OnlyDetectNameInfo detectNameInfo_;
};
class smokerSleepInferenceInstance : public GetInferenceInstance{
public:
    smokerSleepInferenceInstance();
    void Init(const char* kConfigFile) override;
    ~smokerSleepInferenceInstance();
    void getInstance(std::vector<AclLiteThreadParam>& threadTbl, int32_t deviceId,aclrtContext& context, aclrtRunMode& runMode, int rtspNumPerDevice,bool kDisplay, int addrtspNumPerDevice=0);
    const std::string kDetectPreName_ = "smokesleepdetectPre";
    const std::string kDetectPostName_ = "smokesleepdetectPost";
    const std::string kInferName_ = "smokesleepinference";
//    ModelInfo smokesleepinfo_;
//    Detect2clsNameInfo nameInfo_;

};


class yangChenInferenceInstance : public GetInferenceInstance{
public:
    yangChenInferenceInstance();
    void Init(const char* kConfigFile) override;
    ~yangChenInferenceInstance();
    void getInstance(std::vector<AclLiteThreadParam>& threadTbl, int32_t deviceId,aclrtContext& context, aclrtRunMode& runMode, int rtspNumPerDevice,bool kDisplay, int addrtspNumPerDevice=0);
//    const std::string kDetectPreName_ = "smokesleepdetectPre";
//    const std::string kDetectPostName_ = "smokesleepdetectPost";
//    const std::string kInferName_ = "smokesleepinference";
//    ModelInfo yangcheninfo_;
//    OnlyDetectNameInfo detectNameInfo_;

};

class cocoInferenceInstance : public GetInferenceInstance{
public:
    cocoInferenceInstance();
    void Init(const char* kConfigFile) override;
    ~cocoInferenceInstance();
    void getInstance(std::vector<AclLiteThreadParam>& threadTbl, int32_t deviceId,aclrtContext& context, aclrtRunMode& runMode, int rtspNumPerDevice,bool kDisplay,int addrtspNumPerDevice=0);
//    const std::string kDetectPreName_ = "smokesleepdetectPre";
//    const std::string kDetectPostName_ = "smokesleepdetectPost";
//    const std::string kInferName_ = "smokesleepinference";
//    ModelInfo cocopinfo_;
//    OnlyDetectNameInfo detectNameInfo_;

};

class centerfaceMaskInferenceInstance : public GetInferenceInstance{
public:
    centerfaceMaskInferenceInstance();
    void Init(const char* kConfigFile) override;
    ~centerfaceMaskInferenceInstance();
    void getInstance(std::vector<AclLiteThreadParam>& threadTbl, int32_t deviceId,aclrtContext& context, aclrtRunMode& runMode, int rtspNumPerDevice,bool kDisplay, int addrtspNumPerDevice=0);
    const std::string kDetectPreName_ = "centerfacemaskdetectPre";
    const std::string kDetectPostName_ = "centerfacemaskdetectPost";
    const std::string kInferName_ = "centerfacemaskinference";
//    ModelInfo centerfacemaskinfo_;
//    Detect2clsNameInfo nameInfo_;

};




//----------------------------------------------------------------------------------------------------------------------------------------
class Factory{
// 这个位置可以写注册工厂模式，但是由于不确定存不存在线程安全问题（手动狗头）所以先已粗暴的方式写，埋个坑，后人来填,  -_- ! wbq
public:
    GetInferenceInstance* Create(int type){
        switch (type) {
            case INSTANCENAME::CAR_COLOR_INFERENCE_INSTANCE : return new carColorInferenceInstance;
            case INSTANCENAME::HELMET_INFERENCE_INSTANCE : return new helmetInferenceInstance;
            case INSTANCENAME::FIRESMOKE_INFERENCE_INSTANCE : return new fireSmokeInferenceInstance;
            case INSTANCENAME::SMOKER_SLEEP_INFERENCE_INSTANCE : return new smokerSleepInferenceInstance;
            case INSTANCENAME::YANGCHEN_INFERENCE_INSTANCE : return new yangChenInferenceInstance;
            case INSTANCENAME::COCO_PERSON_INFERENCE_INSTANCE : return new cocoInferenceInstance;
            case INSTANCENAME::MASK_INFERENCE_INSTANCE : return new centerfaceMaskInferenceInstance;
            default : break;
        }
    }


};
