//
// Created by 乾三 on 2023/3/1.
//
#include "../include/inferFactory.h"

GetInferenceInstance::GetInferenceInstance() {

}

void GetInferenceInstance::Init(const char *kConfigFile) {
    kConfigFile_ = kConfigFile;
}

GetInferenceInstance::~GetInferenceInstance() {

}

void
GetInferenceInstance::getInstance(std::vector<AclLiteThreadParam> &threadTbl, int32_t deviceId, aclrtContext &context,
                                  aclrtRunMode &runMode, int rtspNumPerDevice, bool kDisplay, int addrtspNumPerDevice) {

}

bool GetInferenceInstance::readOutputTypeConfig(int channelId) {
    std::map<std::string, std::string> config;
    if (!ReadConfig(config, kConfigFile_)) {
        ACLLITE_LOG_ERROR("read config fail in SetDisplay");
    }
    std::string outputType;
    std::string outputTypeKey = "outputType_" + std::to_string(channelId);
    std::map<std::string, std::string>::const_iterator mIter = config.begin();
    for (; mIter != config.end(); ++mIter) {
        if (mIter->first == "URL") {
            kRtspUrl_.assign(mIter->second.c_str());
        }
        if (mIter->first == outputTypeKey) {
            outputType.assign(mIter->second.c_str());
            ACLLITE_LOG_INFO("channelId %d, outputType %s ", channelId, outputType.c_str());
            if (!outputType.empty() && outputType == "rtsp") {
                return true;
            } else {
                return false;
            }
        }
    }
    return false;
}

//----------------------------------------------------------------------------------------------------------------------------------

carColorInferenceInstance::carColorInferenceInstance() {

}

void carColorInferenceInstance::Init(const char* kConfigFile) {
    kConfigFile_ = kConfigFile;
    detectclsNameInfo_.kInferName = "carinference";
    detectclsNameInfo_.kDetectPreName = "cardetectPre";
    detectclsNameInfo_.kDetectPostName = "cardetectPost";
    detectclsNameInfo_.kClassifyPreName = "carclassifyPre";
    detectclsNameInfo_.kClassifyPostName = "carclassifyPost";
    detectclsNameInfo_.kRtspUrl = "carrtspDisplay";
    detectclsNameInfo_.kMqttName = "carcolormqtt";
    modelInfo_.clsifyInputWidth = 224;
    modelInfo_.clsifyInputHeight = 224;
    modelInfo_.modelInputWidth = 640;
    modelInfo_.modelInputHeight = 448;

}

void carColorInferenceInstance::getInstance(std::vector<AclLiteThreadParam> &threadTbl, int32_t deviceId,
                                            aclrtContext &context, aclrtRunMode &runMode, int rtspNumPerDevice,
                                            bool kDisplay,int addrtspNumPerDevice)
{
    AclLiteThreadParam detectPreThreadParam;
    for (int index = 0; index < rtspNumPerDevice + addrtspNumPerDevice; index++) {
        int channelId = deviceId*rtspNumPerDevice+index;
        if (kDisplay) {
            detectPreThreadParam.threadInst =
                new DetectPreprocessThread(kConfigFile_, deviceId, channelId, runMode, detectclsNameInfo_, true);
        } else {
            detectPreThreadParam.threadInst =
                new DetectPreprocessThread(kConfigFile_, deviceId, channelId, runMode, detectclsNameInfo_);
        }
        std::string DetectPreName = detectclsNameInfo_.kDetectPreName + std::to_string(channelId);
        detectPreThreadParam.threadInstName.assign(DetectPreName.c_str());
        detectPreThreadParam.context = context;
        detectPreThreadParam.runMode = runMode;
        threadTbl.push_back(detectPreThreadParam);

        AclLiteThreadParam detectPostThreadParam;
        detectPostThreadParam.threadInst = new DetectPostprocessThread(kConfigFile_,channelId,modelInfo_);
        std::string DetectPostName = detectclsNameInfo_.kDetectPostName + std::to_string(channelId);
        detectPostThreadParam.threadInstName.assign(DetectPostName.c_str());
        detectPostThreadParam.context = context;
        detectPostThreadParam.runMode = runMode;
        threadTbl.push_back(detectPostThreadParam);

        AclLiteThreadParam classifyPreThreadParam;
        classifyPreThreadParam.threadInst = new ClassifyPreprocessThread(runMode,modelInfo_);
        std::string ClassifyPreName = detectclsNameInfo_.kClassifyPreName + std::to_string(channelId);
        classifyPreThreadParam.threadInstName.assign(ClassifyPreName.c_str());
        classifyPreThreadParam.context = context;
        classifyPreThreadParam.runMode = runMode;
        threadTbl.push_back(classifyPreThreadParam);

        AclLiteThreadParam classifyPostThreadParam;
        classifyPostThreadParam.threadInst = new ClassifyPostprocessThread(kConfigFile_, channelId,modelInfo_);
        std::string ClassifyPostName = detectclsNameInfo_.kClassifyPostName + std::to_string(channelId);
        classifyPostThreadParam.threadInstName.assign(ClassifyPostName.c_str());
        classifyPostThreadParam.context = context;
        classifyPostThreadParam.runMode = runMode;
        threadTbl.push_back(classifyPostThreadParam);

        AclLiteThreadParam cocomqttThreadParam;
        cocomqttThreadParam.threadInst = new MqttSendThread("0.0.0.0",1883);
        std::string mqttName = detectclsNameInfo_.kMqttName + std::to_string(channelId);
        cocomqttThreadParam.threadInstName.assign(mqttName.c_str());
        cocomqttThreadParam.context = context;
        cocomqttThreadParam.runMode = runMode;
        threadTbl.push_back(cocomqttThreadParam);

        if (readOutputTypeConfig(channelId)) {
            AclLiteThreadParam rtspDisplayThreadParam;
            rtspDisplayThreadParam.threadInst = new PushRtspThread(kRtspUrl_ + std::to_string(channelId));
            std::string RtspDisplayName = detectclsNameInfo_.kRtspUrl + std::to_string(channelId);
            rtspDisplayThreadParam.threadInstName.assign(RtspDisplayName.c_str());
            rtspDisplayThreadParam.context = context;
            rtspDisplayThreadParam.runMode = runMode;
            threadTbl.push_back(rtspDisplayThreadParam);
        }
    }

    AclLiteThreadParam InferParam;
    InferParam.threadInst = new InferenceThread(runMode,modelInfo_,0);
    std::string InferName = detectclsNameInfo_.kInferName + std::to_string(deviceId);
    InferParam.threadInstName.assign(InferName.c_str());
    InferParam.context = context;
    InferParam.runMode = runMode;
    threadTbl.push_back(InferParam);
}

carColorInferenceInstance::~carColorInferenceInstance() {

}
//----------------------------------------------------------------------------------------------------------------------------------

helmetInferenceInstance::helmetInferenceInstance() {

}

void helmetInferenceInstance::Init(const char *kConfigFile) {
    kConfigFile_ = kConfigFile;
    modelInfo_.classnum = 3;
    modelInfo_.conf = 0.5;
    modelInfo_.Label = { "person", "nohat", "hat"};
    detectNameInfo_.kInferName = "helmetinference";
    detectNameInfo_.kDetectPreName = "helmetdetectPre";
    detectNameInfo_.kDetectPostName = "helmetdetectPost";
    detectNameInfo_.kRtspUrl = "helmetrtspDisplay";
    detectNameInfo_.kMqttName = "helmetmqtt";
    modelInfo_.activateLabel = {1};


}

helmetInferenceInstance::~helmetInferenceInstance() {

}

void helmetInferenceInstance::getInstance(std::vector<AclLiteThreadParam> &threadTbl, int32_t deviceId,
                                          aclrtContext &context, aclrtRunMode &runMode, int rtspNumPerDevice,
                                          bool kDisplay,int addrtspNumPerDevice) {
    AclLiteThreadParam detectPreThreadParam;
    for (int index = 0; index < rtspNumPerDevice + addrtspNumPerDevice; index++) {
        int channelId = deviceId*rtspNumPerDevice+index;
        if (kDisplay) {
            detectPreThreadParam.threadInst =
                    new helmetPreprocessThread(kConfigFile_, deviceId, channelId, runMode, detectNameInfo_,true,1);
        } else {
            detectPreThreadParam.threadInst =
                    new helmetPreprocessThread(kConfigFile_, deviceId, channelId, runMode,detectNameInfo_,false,1);
        }
        std::string DetectPreName = detectNameInfo_.kDetectPreName + std::to_string(channelId);
        detectPreThreadParam.threadInstName.assign(DetectPreName.c_str());
        detectPreThreadParam.context = context;
        detectPreThreadParam.runMode = runMode;
        threadTbl.push_back(detectPreThreadParam);

        AclLiteThreadParam detectPostThreadParam;
        detectPostThreadParam.threadInst = new onlyDetectPostprocessThread(kConfigFile_, channelId, modelInfo_);
        std::string DetectPostName = detectNameInfo_.kDetectPostName + std::to_string(channelId);
        detectPostThreadParam.threadInstName.assign(DetectPostName.c_str());
        detectPostThreadParam.context = context;
        detectPostThreadParam.runMode = runMode;
        threadTbl.push_back(detectPostThreadParam);

        AclLiteThreadParam cocomqttThreadParam;
        cocomqttThreadParam.threadInst = new MqttSendThread("0.0.0.0",1883);
        std::string mqttName = detectNameInfo_.kMqttName + std::to_string(channelId);
        cocomqttThreadParam.threadInstName.assign(mqttName.c_str());
        cocomqttThreadParam.context = context;
        cocomqttThreadParam.runMode = runMode;
        threadTbl.push_back(cocomqttThreadParam);


        if (readOutputTypeConfig(channelId)) {
            AclLiteThreadParam rtspDisplayThreadParam;
            rtspDisplayThreadParam.threadInst = new PushRtspThread(kRtspUrl_ + std::to_string(channelId));
            std::string RtspDisplayName = detectNameInfo_.kRtspUrl + std::to_string(channelId);
            rtspDisplayThreadParam.threadInstName.assign(RtspDisplayName.c_str());
            rtspDisplayThreadParam.context = context;
            rtspDisplayThreadParam.runMode = runMode;
            threadTbl.push_back(rtspDisplayThreadParam);
        }
    }

    AclLiteThreadParam InferParam;
    InferParam.threadInst = new helmetDetectInferenceThread(runMode,INSTANCENAME::HELMET_INFERENCE_INSTANCE); // helmetDetectInferenceThread 为仅检测的infer线程。
    std::string InferName = detectNameInfo_.kInferName + std::to_string(deviceId);
    InferParam.threadInstName.assign(InferName.c_str());
    InferParam.context = context;
    InferParam.runMode = runMode;
    threadTbl.push_back(InferParam);

}
//----------------------------------------------------------------------------------------------------------------------------------

fireSmokeInferenceInstance::fireSmokeInferenceInstance() {

}

void fireSmokeInferenceInstance::Init(const char *kConfigFile) {
    kConfigFile_ = kConfigFile;
    modelInfo_.modelInputWidth = 640;
    modelInfo_.modelInputHeight = 448;
    modelInfo_.classnum = 2;
    modelInfo_.conf = 0.4;
    modelInfo_.totalBox = 17640;
    modelInfo_.Label = { "frie", "smoke"};
    detectNameInfo_.kInferName = "firesmokeinference";
    detectNameInfo_.kDetectPreName = "firesmokedetectPre";
    detectNameInfo_.kDetectPostName = "firesmokedetectPost";
    detectNameInfo_.kRtspUrl = "firesmokespDisplay";
    detectNameInfo_.kMqttName = "firesmokemqtt";



}

fireSmokeInferenceInstance::~fireSmokeInferenceInstance() {

}

void fireSmokeInferenceInstance::getInstance(std::vector<AclLiteThreadParam> &threadTbl, int32_t deviceId,
                                          aclrtContext &context, aclrtRunMode &runMode, int rtspNumPerDevice,
                                          bool kDisplay,int addrtspNumPerDevice) {
    AclLiteThreadParam detectPreThreadParam;
    for (int index = 0; index < rtspNumPerDevice + addrtspNumPerDevice; index++) {
        int channelId = deviceId*rtspNumPerDevice+index;
        if (kDisplay) {
            detectPreThreadParam.threadInst =
                    new helmetPreprocessThread(kConfigFile_, deviceId, channelId, runMode, detectNameInfo_, true,0);
        } else {
            detectPreThreadParam.threadInst =
                    new helmetPreprocessThread(kConfigFile_, deviceId, channelId, runMode, detectNameInfo_, false,0);
        }
        std::string DetectPreName = detectNameInfo_.kDetectPreName + std::to_string(channelId);
        detectPreThreadParam.threadInstName.assign(DetectPreName.c_str());
        detectPreThreadParam.context = context;
        detectPreThreadParam.runMode = runMode;
        threadTbl.push_back(detectPreThreadParam);

        AclLiteThreadParam detectPostThreadParam;
        detectPostThreadParam.threadInst = new onlyDetectPostprocessThread(kConfigFile_, channelId, modelInfo_);
        std::string DetectPostName = detectNameInfo_.kDetectPostName + std::to_string(channelId);
        detectPostThreadParam.threadInstName.assign(DetectPostName.c_str());
        detectPostThreadParam.context = context;
        detectPostThreadParam.runMode = runMode;
        threadTbl.push_back(detectPostThreadParam);

        AclLiteThreadParam cocomqttThreadParam;
        cocomqttThreadParam.threadInst = new MqttSendThread("0.0.0.0",1883);
        std::string mqttName = detectNameInfo_.kMqttName + std::to_string(channelId);
        cocomqttThreadParam.threadInstName.assign(mqttName.c_str());
        cocomqttThreadParam.context = context;
        cocomqttThreadParam.runMode = runMode;
        threadTbl.push_back(cocomqttThreadParam);


        if (readOutputTypeConfig(channelId)) {
            AclLiteThreadParam rtspDisplayThreadParam;
            rtspDisplayThreadParam.threadInst = new PushRtspThread(kRtspUrl_ + std::to_string(channelId));
            std::string RtspDisplayName = detectNameInfo_.kRtspUrl + std::to_string(channelId);
            rtspDisplayThreadParam.threadInstName.assign(RtspDisplayName.c_str());
            rtspDisplayThreadParam.context = context;
            rtspDisplayThreadParam.runMode = runMode;
            threadTbl.push_back(rtspDisplayThreadParam);
        }
    }

    AclLiteThreadParam InferParam;
    InferParam.threadInst = new helmetDetectInferenceThread(runMode,INSTANCENAME::FIRESMOKE_INFERENCE_INSTANCE); // helmetDetectInferenceThread 为仅检测的infer线程。
    std::string InferName = detectNameInfo_.kInferName + std::to_string(deviceId);
    InferParam.threadInstName.assign(InferName.c_str());
    InferParam.context = context;
    InferParam.runMode = runMode;
    threadTbl.push_back(InferParam);

}
//----------------------------------------------------------------------------------------------------------------------------------


smokerSleepInferenceInstance::smokerSleepInferenceInstance() {

}

void smokerSleepInferenceInstance::Init(const char *kConfigFile) {
    kConfigFile_ = kConfigFile;
    modelInfo_.modelInputWidth = 640;
    modelInfo_.modelInputHeight = 448;
    modelInfo_.classnum = 80;
    modelInfo_.conf = 0.4;
    modelInfo_.totalBox = 17640;
    modelInfo_.clsifyInputWidth = 256;
    modelInfo_.clsifyInputHeight = 256;
    detectclsNameInfo_.kInferName = "smokerinference";
//    modelInfo_.Label = { "frie", "smoke"};
    detectclsNameInfo_.kDetectPreName = "smokerdetectPre";
    detectclsNameInfo_.kDetectPostName = "smokerdetectPost";
    detectclsNameInfo_.kClassifyPreName = "smokerclassifyPre";
    detectclsNameInfo_.kClassifyPostName = "smokerclassifyPost";
    detectclsNameInfo_.kRtspUrl = "smokerrtspDisplay";
    detectclsNameInfo_.kMqttName = "smokersleepmqtt";
    modelInfo_.clsLabel = { "calling","normal","sleep","smoking"};
}

smokerSleepInferenceInstance::~smokerSleepInferenceInstance() {

}

void smokerSleepInferenceInstance::getInstance(std::vector<AclLiteThreadParam> &threadTbl, int32_t deviceId,
                                               aclrtContext &context, aclrtRunMode &runMode, int rtspNumPerDevice,
                                               bool kDisplay,int addrtspNumPerDevice)
                                               {

        AclLiteThreadParam detectPreThreadParam;
        for (int index = 0; index < rtspNumPerDevice + addrtspNumPerDevice; index++) {
            int channelId = deviceId*rtspNumPerDevice+index;
            if (kDisplay) {
                detectPreThreadParam.threadInst =
                        new DetectPreprocessThread(kConfigFile_, deviceId, channelId, runMode, detectclsNameInfo_, true);
            } else {
                detectPreThreadParam.threadInst =
                        new DetectPreprocessThread(kConfigFile_, deviceId, channelId, runMode, detectclsNameInfo_);
            }
            std::string DetectPreName = detectclsNameInfo_.kDetectPreName + std::to_string(channelId);
            detectPreThreadParam.threadInstName.assign(DetectPreName.c_str());
            detectPreThreadParam.context = context;
            detectPreThreadParam.runMode = runMode;
            threadTbl.push_back(detectPreThreadParam);

            AclLiteThreadParam detectPostThreadParam;
            detectPostThreadParam.threadInst = new DetectPostprocessThread(kConfigFile_,channelId,modelInfo_);
            std::string DetectPostName = detectclsNameInfo_.kDetectPostName + std::to_string(channelId);
            detectPostThreadParam.threadInstName.assign(DetectPostName.c_str());
            detectPostThreadParam.context = context;
            detectPostThreadParam.runMode = runMode;
            threadTbl.push_back(detectPostThreadParam);

            AclLiteThreadParam classifyPreThreadParam;
            classifyPreThreadParam.threadInst = new ClassifyPreprocessThread(runMode,modelInfo_);
            std::string ClassifyPreName = detectclsNameInfo_.kClassifyPreName + std::to_string(channelId);
            classifyPreThreadParam.threadInstName.assign(ClassifyPreName.c_str());
            classifyPreThreadParam.context = context;
            classifyPreThreadParam.runMode = runMode;
            threadTbl.push_back(classifyPreThreadParam);

            AclLiteThreadParam classifyPostThreadParam;
            classifyPostThreadParam.threadInst = new ClassifyPostprocessThread(kConfigFile_, channelId,modelInfo_);
            std::string ClassifyPostName = detectclsNameInfo_.kClassifyPostName + std::to_string(channelId);
            classifyPostThreadParam.threadInstName.assign(ClassifyPostName.c_str());
            classifyPostThreadParam.context = context;
            classifyPostThreadParam.runMode = runMode;
            threadTbl.push_back(classifyPostThreadParam);

            AclLiteThreadParam cocomqttThreadParam;
            cocomqttThreadParam.threadInst = new MqttSendThread("0.0.0.0",1883);
            std::string mqttName = detectclsNameInfo_.kMqttName + std::to_string(channelId);
            cocomqttThreadParam.threadInstName.assign(mqttName.c_str());
            cocomqttThreadParam.context = context;
            cocomqttThreadParam.runMode = runMode;
            threadTbl.push_back(cocomqttThreadParam);

            if (readOutputTypeConfig(channelId)) {
                AclLiteThreadParam rtspDisplayThreadParam;
                rtspDisplayThreadParam.threadInst = new PushRtspThread(kRtspUrl_ + std::to_string(channelId));
                std::string RtspDisplayName = detectclsNameInfo_.kRtspUrl + std::to_string(channelId);
                rtspDisplayThreadParam.threadInstName.assign(RtspDisplayName.c_str());
                rtspDisplayThreadParam.context = context;
                rtspDisplayThreadParam.runMode = runMode;
                threadTbl.push_back(rtspDisplayThreadParam);
            }
        }

        AclLiteThreadParam InferParam;
        InferParam.threadInst = new InferenceThread(runMode,modelInfo_,3);
        std::string InferName = detectclsNameInfo_.kInferName + std::to_string(deviceId);
        InferParam.threadInstName.assign(InferName.c_str());
        InferParam.context = context;
        InferParam.runMode = runMode;
        threadTbl.push_back(InferParam);
    }
//----------------------------------------------------------------------------------------------------------------------------------

yangChenInferenceInstance::yangChenInferenceInstance() {

}

void yangChenInferenceInstance::Init(const char *kConfigFile) {
    kConfigFile_ = kConfigFile;
    modelInfo_.modelInputWidth = 640;
    modelInfo_.modelInputHeight = 448;
    modelInfo_.classnum = 1;
    modelInfo_.conf = 0.4;
    modelInfo_.totalBox = 17640;
    modelInfo_.Label = {"yangchen"};
    detectNameInfo_.kInferName = "yangCheninference";
    detectNameInfo_.kDetectPreName = "yangChendetectPre";
    detectNameInfo_.kDetectPostName = "yangChendetectPost";
    detectNameInfo_.kRtspUrl = "yangChenDisplay";
    detectNameInfo_.kMqttName = "yangchenmqtt";


}

yangChenInferenceInstance::~yangChenInferenceInstance() {

}

void yangChenInferenceInstance::getInstance(std::vector<AclLiteThreadParam> &threadTbl, int32_t deviceId,
                                             aclrtContext &context, aclrtRunMode &runMode, int rtspNumPerDevice,
                                             bool kDisplay,int addrtspNumPerDevice) {
    AclLiteThreadParam detectPreThreadParam;
    for (int index = 0; index < rtspNumPerDevice + addrtspNumPerDevice; index++) {
        int channelId = deviceId*rtspNumPerDevice+index;
        if (kDisplay) {
            detectPreThreadParam.threadInst =
                    new helmetPreprocessThread(kConfigFile_, deviceId, channelId, runMode, detectNameInfo_, true,0);
        } else {
            detectPreThreadParam.threadInst =
                    new helmetPreprocessThread(kConfigFile_, deviceId, channelId, runMode, detectNameInfo_, false,0);
        }
        std::string DetectPreName = detectNameInfo_.kDetectPreName + std::to_string(channelId);
        detectPreThreadParam.threadInstName.assign(DetectPreName.c_str());
        detectPreThreadParam.context = context;
        detectPreThreadParam.runMode = runMode;
        threadTbl.push_back(detectPreThreadParam);

        AclLiteThreadParam detectPostThreadParam;
        detectPostThreadParam.threadInst = new onlyDetectPostprocessThread(kConfigFile_, channelId, modelInfo_);
        std::string DetectPostName = detectNameInfo_.kDetectPostName + std::to_string(channelId);
        detectPostThreadParam.threadInstName.assign(DetectPostName.c_str());
        detectPostThreadParam.context = context;
        detectPostThreadParam.runMode = runMode;
        threadTbl.push_back(detectPostThreadParam);

        AclLiteThreadParam cocomqttThreadParam;
        cocomqttThreadParam.threadInst = new MqttSendThread("0.0.0.0",1883);
        std::string mqttName = detectNameInfo_.kMqttName + std::to_string(channelId);
        cocomqttThreadParam.threadInstName.assign(mqttName.c_str());
        cocomqttThreadParam.context = context;
        cocomqttThreadParam.runMode = runMode;
        threadTbl.push_back(cocomqttThreadParam);

        if (readOutputTypeConfig(channelId)) {
            AclLiteThreadParam rtspDisplayThreadParam;
            rtspDisplayThreadParam.threadInst = new PushRtspThread(kRtspUrl_ + std::to_string(channelId));
            std::string RtspDisplayName = detectNameInfo_.kRtspUrl + std::to_string(channelId);
            rtspDisplayThreadParam.threadInstName.assign(RtspDisplayName.c_str());
            rtspDisplayThreadParam.context = context;
            rtspDisplayThreadParam.runMode = runMode;
            threadTbl.push_back(rtspDisplayThreadParam);
        }
    }

    AclLiteThreadParam InferParam;
    InferParam.threadInst = new helmetDetectInferenceThread(runMode,INSTANCENAME::YANGCHEN_INFERENCE_INSTANCE); // helmetDetectInferenceThread 为仅检测的infer线程。
    std::string InferName = detectNameInfo_.kInferName + std::to_string(deviceId);
    InferParam.threadInstName.assign(InferName.c_str());
    InferParam.context = context;
    InferParam.runMode = runMode;
    threadTbl.push_back(InferParam);

}


//----------------------------------------------------------------------------------------------------------------------------------
cocoInferenceInstance::cocoInferenceInstance() {

}

void cocoInferenceInstance::Init(const char *kConfigFile) {
    kConfigFile_ = kConfigFile;
    modelInfo_.modelInputWidth = 640;
    modelInfo_.modelInputHeight = 448;
    modelInfo_.classnum = 80;
    modelInfo_.conf = 0.35;
    modelInfo_.totalBox = 17640;
    detectNameInfo_.kInferName = "cocopinference";
    detectNameInfo_.kDetectPreName = "cocopdetectPre";
    detectNameInfo_.kDetectPostName = "cocopdetectPost";
    detectNameInfo_.kRtspUrl = "cocopDisplay";
    detectNameInfo_.kMqttName = "cocopmqtt";
    modelInfo_.activateLabel = {0};

}

cocoInferenceInstance::~cocoInferenceInstance() {

}

void cocoInferenceInstance::getInstance(std::vector<AclLiteThreadParam> &threadTbl, int32_t deviceId,
                                            aclrtContext &context, aclrtRunMode &runMode, int rtspNumPerDevice,
                                            bool kDisplay,int addrtspNumPerDevice) {
    AclLiteThreadParam detectPreThreadParam;
    for (int index = 0; index < rtspNumPerDevice + addrtspNumPerDevice; index++) {
        int channelId = deviceId*rtspNumPerDevice+index;
        if (kDisplay) {
            detectPreThreadParam.threadInst =
                    new helmetPreprocessThread(kConfigFile_, deviceId, channelId, runMode, detectNameInfo_, true,0);
        } else {
            detectPreThreadParam.threadInst =
                    new helmetPreprocessThread(kConfigFile_, deviceId, channelId, runMode, detectNameInfo_, false,0);
        }
        std::string DetectPreName = detectNameInfo_.kDetectPreName + std::to_string(channelId);
        detectPreThreadParam.threadInstName.assign(DetectPreName.c_str());
        detectPreThreadParam.context = context;
        detectPreThreadParam.runMode = runMode;
        threadTbl.push_back(detectPreThreadParam);

        AclLiteThreadParam detectPostThreadParam;
        detectPostThreadParam.threadInst = new onlyDetectPostprocessThread(kConfigFile_, channelId, modelInfo_);
        std::string DetectPostName = detectNameInfo_.kDetectPostName + std::to_string(channelId);
        detectPostThreadParam.threadInstName.assign(DetectPostName.c_str());
        detectPostThreadParam.context = context;
        detectPostThreadParam.runMode = runMode;
        threadTbl.push_back(detectPostThreadParam);

        AclLiteThreadParam cocomqttThreadParam;
        cocomqttThreadParam.threadInst = new MqttSendThread("0.0.0.0",1883);
        std::string mqttName = detectNameInfo_.kMqttName + std::to_string(channelId);
        cocomqttThreadParam.threadInstName.assign(mqttName.c_str());
        cocomqttThreadParam.context = context;
        cocomqttThreadParam.runMode = runMode;
        threadTbl.push_back(cocomqttThreadParam);


        if (readOutputTypeConfig(channelId)) {
            AclLiteThreadParam rtspDisplayThreadParam;
            rtspDisplayThreadParam.threadInst = new PushRtspThread(kRtspUrl_ + std::to_string(channelId));
            std::string RtspDisplayName = detectNameInfo_.kRtspUrl + std::to_string(channelId);
            rtspDisplayThreadParam.threadInstName.assign(RtspDisplayName.c_str());
            rtspDisplayThreadParam.context = context;
            rtspDisplayThreadParam.runMode = runMode;
            threadTbl.push_back(rtspDisplayThreadParam);
        }
    }

    AclLiteThreadParam InferParam;
    InferParam.threadInst = new helmetDetectInferenceThread(runMode,INSTANCENAME::COCO_PERSON_INFERENCE_INSTANCE); // helmetDetectInferenceThread 为仅检测的infer线程。
    std::string InferName = detectNameInfo_.kInferName + std::to_string(deviceId);
    InferParam.threadInstName.assign(InferName.c_str());
    InferParam.context = context;
    InferParam.runMode = runMode;
    threadTbl.push_back(InferParam);

}

//----------------------------------------------------------------------------------------------------------------------------------

centerfaceMaskInferenceInstance::centerfaceMaskInferenceInstance() {

}

void centerfaceMaskInferenceInstance::Init(const char *kConfigFile) {
    kConfigFile_ = kConfigFile;
    modelInfo_.modelInputWidth = 640;
    modelInfo_.modelInputHeight = 448;
    modelInfo_.classnum = 1;
    modelInfo_.conf = 0.4;
    modelInfo_.totalBox = 17640;
    modelInfo_.clsifyInputWidth = 224;
    modelInfo_.clsifyInputHeight = 224;
    detectclsNameInfo_.kInferName = "centerfaceinference";
    modelInfo_.Label = { "face"};
    detectclsNameInfo_.kDetectPreName = "centerfacedetectPre";
    detectclsNameInfo_.kDetectPostName = "centerfacedetectPost";
    detectclsNameInfo_.kClassifyPreName = "centerfaceclassifyPre";
    detectclsNameInfo_.kClassifyPostName = "centerfaceclassifyPost";
    detectclsNameInfo_.kRtspUrl = "centerfacertspDisplay";
    detectclsNameInfo_.kMqttName = "centerfacesleepmqtt";
    modelInfo_.clsLabel = { "nomask","mask",};
}

centerfaceMaskInferenceInstance::~centerfaceMaskInferenceInstance() {

}

void centerfaceMaskInferenceInstance::getInstance(std::vector<AclLiteThreadParam> &threadTbl, int32_t deviceId,
                                               aclrtContext &context, aclrtRunMode &runMode, int rtspNumPerDevice,
                                               bool kDisplay,int addrtspNumPerDevice)
{

    AclLiteThreadParam detectPreThreadParam;
    for (int index = 0; index < rtspNumPerDevice + addrtspNumPerDevice; index++) {
        int channelId = deviceId*rtspNumPerDevice+index;
        if (kDisplay) {
            detectPreThreadParam.threadInst =
                    new DetectPreprocessThread(kConfigFile_, deviceId, channelId, runMode, detectclsNameInfo_, true);
        } else {
            detectPreThreadParam.threadInst =
                    new DetectPreprocessThread(kConfigFile_, deviceId, channelId, runMode, detectclsNameInfo_);
        }
        std::string DetectPreName = detectclsNameInfo_.kDetectPreName + std::to_string(channelId);
        detectPreThreadParam.threadInstName.assign(DetectPreName.c_str());
        detectPreThreadParam.context = context;
        detectPreThreadParam.runMode = runMode;
        threadTbl.push_back(detectPreThreadParam);

        AclLiteThreadParam detectPostThreadParam;
        detectPostThreadParam.threadInst = new centerfacePostprocessThread(kConfigFile_,channelId,modelInfo_);
        std::string DetectPostName = detectclsNameInfo_.kDetectPostName + std::to_string(channelId);
        detectPostThreadParam.threadInstName.assign(DetectPostName.c_str());
        detectPostThreadParam.context = context;
        detectPostThreadParam.runMode = runMode;
        threadTbl.push_back(detectPostThreadParam);

        AclLiteThreadParam classifyPreThreadParam;
        classifyPreThreadParam.threadInst = new ClassifyPreprocessThread(runMode,modelInfo_);
        std::string ClassifyPreName = detectclsNameInfo_.kClassifyPreName + std::to_string(channelId);
        classifyPreThreadParam.threadInstName.assign(ClassifyPreName.c_str());
        classifyPreThreadParam.context = context;
        classifyPreThreadParam.runMode = runMode;
        threadTbl.push_back(classifyPreThreadParam);

        AclLiteThreadParam classifyPostThreadParam;
        classifyPostThreadParam.threadInst = new ClassifyPostprocessThread(kConfigFile_, channelId,modelInfo_);
        std::string ClassifyPostName = detectclsNameInfo_.kClassifyPostName + std::to_string(channelId);
        classifyPostThreadParam.threadInstName.assign(ClassifyPostName.c_str());
        classifyPostThreadParam.context = context;
        classifyPostThreadParam.runMode = runMode;
        threadTbl.push_back(classifyPostThreadParam);

        AclLiteThreadParam cocomqttThreadParam;
        cocomqttThreadParam.threadInst = new MqttSendThread("0.0.0.0",1883);
        std::string mqttName = detectclsNameInfo_.kMqttName + std::to_string(channelId);
        cocomqttThreadParam.threadInstName.assign(mqttName.c_str());
        cocomqttThreadParam.context = context;
        cocomqttThreadParam.runMode = runMode;
        threadTbl.push_back(cocomqttThreadParam);

        if (readOutputTypeConfig(channelId)) {
            AclLiteThreadParam rtspDisplayThreadParam;
            rtspDisplayThreadParam.threadInst = new PushRtspThread(kRtspUrl_ + std::to_string(channelId));
            std::string RtspDisplayName = detectclsNameInfo_.kRtspUrl + std::to_string(channelId);
            rtspDisplayThreadParam.threadInstName.assign(RtspDisplayName.c_str());
            rtspDisplayThreadParam.context = context;
            rtspDisplayThreadParam.runMode = runMode;
            threadTbl.push_back(rtspDisplayThreadParam);
        }
    }

    AclLiteThreadParam InferParam;
    InferParam.threadInst = new InferenceThread(runMode,modelInfo_,6);
    std::string InferName = detectclsNameInfo_.kInferName + std::to_string(deviceId);
    InferParam.threadInstName.assign(InferName.c_str());
    InferParam.context = context;
    InferParam.runMode = runMode;
    threadTbl.push_back(InferParam);
}

//----------------------------------------------------------------------------------------------------------------------------------