/**
* Copyright (c) Huawei Technologies Co., Ltd. 2020-2022. All rights reserved.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at

* http://www.apache.org/licenses/LICENSE-2.0

* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.

* File main.cpp
* Description: dvpp sample main func
*/

#include <iostream>
#include <cstdlib>
#include <dirent.h>
#include <sys/time.h>
#include <regex>
#include <map>
#include "AclLiteResource.h"
#include "AclLiteApp.h"
#include "AclLiteThread.h"
#include "AclLiteType.h"
#include "VideoCapture.h"
#include "AclLiteUtils.h"
#include "CarParams.h"
//#include "inference/inference.h"
//#include "detectPreprocess/detectPreprocess.h"
//#include "detectPostprocess/detectPostprocess.h"
//#include "classifyPreprocess/classifyPreprocess.h"
//#include "classifyPostprocess/classifyPostprocess.h"
#include "include/inferFactory.h"
#ifdef USE_PRESENT
#include "presentagentDisplay/presentagentDisplay.h"
#endif

#include "pushrtsp/pushrtspthread.h"
#include "include/cmdline.h"
#define NUM (1)
#define INVALID (-1)
using namespace std;

#ifdef USE_PRESENT
using namespace ascend::presenter;
#endif

struct timespec time0 = {0, 0};
struct timespec time1 = {0, 0};

namespace {
uint32_t kExitCount = 0;
bool kDisplay = false;
char* kConfigFile = "../scripts/params.conf";
const char* kPresentagentFile = "../scripts/present_start.conf";
const string kRegexDeviceNum = "^device_num+$";
const string kRegexRtspNumPerDevice = "^RtspNumPerDevice+$";
string kRtspUrl;

#ifdef USE_PRESENT
Channel* kPresenterChannel;
#endif

aclrtContext context;
aclrtRunMode runMode;
vector<aclrtContext> kContext;
}

int MainThreadProcess(uint32_t msgId,
                      shared_ptr<void> msgData, void* userData)
{
    if (msgId == MSG_APP_EXIT) {
        kExitCount--;
    }
    if (!kExitCount) {
        AclLiteApp& app = GetAclLiteAppInstance();
        app.WaitEnd();
        ACLLITE_LOG_INFO("Receive exit message, exit now");
    }
    return ACLLITE_OK;
}

#ifdef USE_PRESENT
AclLiteError InitPresentAgent()
{
    PresenterErrorCode ret = OpenChannelByConfig(kPresenterChannel, kPresentagentFile);
    if (ret != PresenterErrorCode::kNone) {
        ACLLITE_LOG_ERROR("Open channel failed, error %d", (int)ret);
        return ACLLITE_ERROR;
    }
    ACLLITE_LOG_INFO("Present Agent open success");
    return ACLLITE_OK;
}
#endif

AclLiteError ParseConfig(uint32_t& deviceNum, uint32_t& rtspNumPerDevice, const char* configFile)
{
    map<string, string> config;
    if (!ReadConfig(config, configFile)) {
        return ACLLITE_ERROR;
    }

    regex deviceNumRegex(kRegexDeviceNum.c_str());
    regex RtspNumPerDeviceRegex(kRegexRtspNumPerDevice.c_str());
    map<string, string>::const_iterator mIter = config.begin();
    for (; mIter != config.end(); ++mIter) {
        if (regex_match(mIter->first, deviceNumRegex)) {
            deviceNum = stoi(mIter->second);
            ACLLITE_LOG_INFO("Data config item: %s=%s",
                             mIter->first.c_str(), mIter->second.c_str());
        } else if (regex_match(mIter->first, RtspNumPerDeviceRegex)) {
            rtspNumPerDevice = stoi(mIter->second);
            ACLLITE_LOG_INFO("Data config item: %s=%s",
                             mIter->first.c_str(), mIter->second.c_str());
        }
    }
    return ACLLITE_OK;
}

//void SetDisplay()
//{
//    if (kDisplay) {
//        return;
//    }
//    uint32_t deviceNum;
//    uint32_t rtspNumPerDevice;
//    AclLiteError ret = ParseConfig(deviceNum, rtspNumPerDevice);
//    if (ret != ACLLITE_OK) {
//        ACLLITE_LOG_ERROR("Parse config fail in SetDisplay");
//    }
//    map<string, string> config;
//    if (!ReadConfig(config, kConfigFile)) {
//        ACLLITE_LOG_ERROR("read config fail in SetDisplay");
//    }
//    string outputTypeKey, outputType;
//    uint32_t presentAgentNum = 0;
//    for(int32_t i = 0; i < deviceNum; i++) {
//        string outputTypeKey = "outputType_" + to_string(i);
//        map<string, string>::const_iterator mIter = config.begin();
//        for (; mIter != config.end(); ++mIter) {
//            if (mIter->first == outputTypeKey) {
//                outputType.assign(mIter->second.c_str());
//            }
//        }
//        if (!outputType.empty() && outputType == "presentagent") {
//            kDisplay = true;
//            ACLLITE_LOG_INFO("Set kDisplay true");
//        }
//        ACLLITE_LOG_INFO("Set kDisplay false");
//    }
//}


void CreateALLThreadInstance(vector<AclLiteThreadParam>& threadTbl, AclLiteResource& aclDev,int programType,const char* configFile)
{
    uint32_t deviceNum;
    uint32_t rtspNumPerDevice;
    runMode = aclDev.GetRunMode();

    Factory* instancefactoryApp = new Factory;
    auto instanceApp = instancefactoryApp->Create(programType);
    instanceApp->Init(configFile);
    AclLiteError ret = ParseConfig(deviceNum, rtspNumPerDevice,configFile);
    if (ret != ACLLITE_OK) {
        return;
    }
//    kExitCount = deviceNum * rtspNumPerDevice;
    kExitCount = rtspNumPerDevice;
    int quotient = rtspNumPerDevice / deviceNum;
    int remainder = rtspNumPerDevice % deviceNum;

    for (int32_t i=0; i < deviceNum; i++) {
        int addProcess = 0;
        if (i == (deviceNum - 1)){
            addProcess = remainder;
        }
        context = aclDev.GetContextByDevice(i);
        if (context == nullptr) {
            ACLLITE_LOG_ERROR("Get acl context in device %d failed", i);
            return;
        }
        kContext.push_back(context);
        instanceApp->getInstance(threadTbl, i, context, runMode, quotient,kDisplay,addProcess);
//        CreateThreadInstance(threadTbl, i, context, runMode, rtspNumPerDevice);
    }

    #ifdef USE_PRESENT
    if (kDisplay) {
        AclLiteThreadParam presentAgentDisplayThreadParam;
        presentAgentDisplayThreadParam.threadInst = new PresentAgentDisplayThread(kPresenterChannel);
        presentAgentDisplayThreadParam.threadInstName.assign(kPresentAgentDisplayName.c_str());
        presentAgentDisplayThreadParam.context = context;
        presentAgentDisplayThreadParam.runMode = runMode;
        threadTbl.push_back(presentAgentDisplayThreadParam);
    }
    #endif
}

void ExitApp(AclLiteApp& app, vector<AclLiteThreadParam>& threadTbl)
{
    for (int i = 0; i < threadTbl.size(); i++) {
        aclrtSetCurrentContext(threadTbl[i].context);
        delete threadTbl[i].threadInst;
    }
    app.Exit();

    for (int i = 0; i < kContext.size(); i++) {
        aclrtDestroyContext(kContext[i]);
        aclrtResetDevice(i);
    }
}

void StartApp(AclLiteResource& aclDev,int programType,const char* configFile)
{
    vector<AclLiteThreadParam> threadTbl;
//    int programType = 2;



    CreateALLThreadInstance(threadTbl, aclDev,programType,configFile);
    AclLiteApp& app = CreateAclLiteAppInstance();
    AclLiteError ret = app.Start(threadTbl);
    if (ret != ACLLITE_OK) {
        ACLLITE_LOG_ERROR("Start app failed, error %d", ret);
        ExitApp(app, threadTbl);
        return;
    }

    // time print
    clock_gettime(CLOCK_REALTIME, &time0);
    for (int i = 0; i < threadTbl.size(); i++) {
        ret = SendMessage(threadTbl[i].threadInstId, MSG_APP_START, nullptr);
    }
    app.Wait(MainThreadProcess, nullptr);

    clock_gettime(CLOCK_REALTIME, &time1);
    cout<<"/*****************************/"<<endl;
    cout << "process time is: " << (time1.tv_sec - time0.tv_sec)*1000 +
        (time1.tv_nsec - time0.tv_nsec)/1000000 << "ms" << endl;
    cout<<"/*****************************/"<<endl;
    ExitApp(app, threadTbl);

    return;
}

int main(int argc, char** argv)
{
    cmdline::parser configArgs;
    configArgs.add<std::string>("config",'c',"配置文件地址", true,"");
    configArgs.add<int>("ptype",'p',"算法id"
                                    " 汽车颜色检测 id = 0"
                                    " 安全帽检测 id = 1"
                                    " 火焰烟雾检测 id = 2"
                                    " 吸烟睡岗检测 id = 3"
                                    " coco 人员检测 id = 4"
                                    " 扬尘检测 id = 5"
                                    " 口罩检测 id = 6", true,1);

//    const int CAR_COLOR_INFERENCE_INSTANCE = 0;
//    const int HELMET_INFERENCE_INSTANCE = 1;
//    const int FIRESMOKE_INFERENCE_INSTANCE = 2;
//    const int SMOKER_SLEEP_INFERENCE_INSTANCE = 3;
//    const int COCO_PERSON_INFERENCE_INSTANCE = 4;
//    const int YANGCHEN_INFERENCE_INSTANCE = 5;
//    const int MASK_INFERENCE_INSTANCE = 6;

    configArgs.parse_check(argc,argv);

    AclLiteResource aclDev = AclLiteResource();
    AclLiteError ret = aclDev.Init();
    if (ret != ACLLITE_OK) {
        ACLLITE_LOG_ERROR("Init app failed");
    }
//    SetDisplay();
    int programType = configArgs.get<int>("ptype");
    string conf = configArgs.get<std::string>("config");
    const char * configer = conf.c_str();
//    kConfigFile = conf.c_str();
//    kConfigFile = configArgs.get<std::string>("config");
    #ifdef USE_PRESENT
    if (kDisplay) {
        ret = InitPresentAgent();
        if (ret != ACLLITE_OK) {
            ACLLITE_LOG_ERROR("Init present agent failed");
        }
    }
    #endif

    StartApp(aclDev,programType,configer);
    return ACLLITE_OK;
}
