//
// Created by 乾三 on 2023/3/31.
//

#include "../include/mqttsend.h"

#include "AclLiteApp.h"
#include <iostream>
#include <chrono>
#include <ctime>
#include <sys/stat.h>
#include <unistd.h>
#include <experimental/filesystem>
using namespace std;
namespace fs = std::experimental::filesystem;

MqttSendThread::MqttSendThread(const std::string& host, const int port)
        : host_(host)
        , port_(port)
{


}

MqttSendThread::~MqttSendThread()
{
    if (connected_) {
        mosquitto_destroy(mosq_);
    }
}

AclLiteError MqttSendThread::Init() {

    mosq_ = mosquitto_new(nullptr, true, nullptr);
    if (mosquitto_connect(mosq_, host_.c_str(), port_, keep_alive_) != MOSQ_ERR_SUCCESS) {
//        std::cerr <<  << std::endl;
        ACLLITE_LOG_ERROR("connect mqtt error!!");
    } else {
        connected_ = true;
        ACLLITE_LOG_INFO("connect mqtt succes ");
    }

    return ACLLITE_OK;
}

bool is_directory_exist(const std::string &path) {
    struct stat info;
    if(stat(path.c_str(), &info) != 0) {
        return false;
    } else {
        return (info.st_mode & S_IFDIR) != 0;
    }
}

bool create_directory(const std::string &path) {
    if (mkdir(path.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH) != 0) {
        return false;
    } else {
        return true;
    }
}

AclLiteError MqttSendThread::Process(int msgId, std::shared_ptr<void> msgData) {
    switch(msgId) {
        case MSG_MQTT_SEND:
            SendMsg(static_pointer_cast<ObjDetectDataMsg>(msgData));
            break;
        case MSG_ENCODE_FINISH:
            SendMessage(g_MainThreadId, MSG_APP_EXIT, nullptr);
            break;
        default:
            ACLLITE_LOG_INFO("Present agent display thread ignore msg %d", msgId);
            break;
    }
    return ACLLITE_OK;
}

AclLiteError MqttSendThread::mqtt_publish_handle(std::shared_ptr<ObjDetectDataMsg> objDetectDataMsg)
{

    Json::Value value ;



    std::chrono::system_clock::time_point now = std::chrono::system_clock::now();
    std::time_t now_c = std::chrono::system_clock::to_time_t(now);
    struct tm* parts = std::localtime(&now_c);
    std::string year = std::to_string(parts->tm_year + 1900);
    std::string month = std::to_string(parts->tm_mon + 1);
    std::string day = std::to_string(parts->tm_mday);

    std::string path = "/history_pic/" + year + "/" + month + "/" + day;
    std::error_code ec;
    if (!is_directory_exist(path)) {
        fs::create_directories(path,ec);
        if (ec.value() == 0) {
            ACLLITE_LOG_INFO("成功创建目录 %s", path.c_str());
        } else {
            ACLLITE_LOG_ERROR("创建目录失败 %s",path.c_str());

        }
    }

    path =  path + "/" + to_string(now_c) + '_'+ to_string(objDetectDataMsg->channelId) + "_" + to_string(objDetectDataMsg->frameNum) + ".jpg";


//        value["pic_data"]=Json::Value(imgbase64);
    value["sort"]=Json::Value(objDetectDataMsg->channelId);
//    value["equipment_nl"]=Json::Value("no001");
    value["timeStamp"]= Json::Value(to_string(now_c));

    Json::Value wb;


//    cout<<"[rtsp video "<<objDetectDataMsg->channelId<<"]: { ";
    for (int i = 0; i < objDetectDataMsg->objInfo.size(); ++i) {
        Json::Value info;
        info["x1"] = objDetectDataMsg->objInfo[i].rectangle.lt.x;
        info["y1"] = objDetectDataMsg->objInfo[i].rectangle.lt.y;
        info["x2"] = objDetectDataMsg->objInfo[i].rectangle.rb.x;
        info["y2"] = objDetectDataMsg->objInfo[i].rectangle.rb.y;
        std::string classname = objDetectDataMsg->objInfo[i].detect_result;
        info["label"] = classname;
        wb.append(info);
    }
    value["result"]=wb;
    if (wb.size()!= 0){
        value["pictureAddress"]=Json::Value(path);
        cv::imwrite(path.c_str(), objDetectDataMsg->frame);
    } else{
        value["pictureAddress"]=Json::Value("");
    }




    std::string strResult = value.toStyledString();


    int mosquitto_state = mosquitto_publish(mosq_, nullptr, mesg_pub_topic_.c_str(),
                                            static_cast<int>(strResult.length()) + 1,
                                            strResult.c_str(), 0, 0) ;
    if ( mosquitto_state != MOSQ_ERR_SUCCESS) {

        ACLLITE_LOG_ERROR("MQTT publish error");
        return ACLLITE_ERROR;
    }

    usleep(10000);
    return ACLLITE_OK;
}

AclLiteError MqttSendThread::SendMsg(std::shared_ptr<ObjDetectDataMsg> objDetectDataMsg) {
    if (objDetectDataMsg->isLastFrame == 1) {
//        g_picToRtsp.FlushEncoder();
        SendMessage(objDetectDataMsg->mqttSendThreadId, MSG_ENCODE_FINISH, nullptr);
        return ACLLITE_OK;
    }

    mqtt_publish_handle(objDetectDataMsg);

    return ACLLITE_OK;
}